/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PCAP_DUMPER_HXX_
# define PCAP_DUMPER_HXX_

# include "dumper.hh"

namespace pcapxx
{

  inline
  dumper::dumper(const std::string&	filename,
		 int32_t		type,
		 int32_t		snaplen):
    file_name_ (filename),
    pkt_count_ (0),
    output_ (new std::ofstream (filename.c_str(), (std::ios::out   |
						   std::ios::trunc |
						   std::ios::binary)))
  {
    internals::file_header h;

    h.magic		= 0xa1b2c3d4;
    h.version_major	= internals::version_major;
    h.version_minor	= internals::version_minor;
    h.thiszone		= 0;
    h.sigfigs		= 0;
    h.snaplen		= snaplen;
    h.linktype		= type;

    if (not *output_.get())
      throw tool::file_error("Could not open file for writing");
    if (not output_->write(reinterpret_cast<char*> (&h), sizeof (h)))
      throw tool::write_error("Could not write PCAP file header");

    marks_.push_back(output_->tellp());
  }

  inline
  void
  dumper::operator () (const frame_descriptor& d)
  {
    return (*this) (d.pcap_header().get(), d.bytes().get());
  }

} // End of namespace pcapxx.

/* dumper::operator () (const pkthdr* h, const void* f)
 *
 *	   and
 *
 * dumper::descriptor() const
 *
 * are implemented in descriptor.hxx.  (I know, that is ugly.  That is
 * to avoid circular dependancy issues.)
 */
# include <wipal/pcap/descriptor.hh>

#endif // ! PCAP_DESCRITPOR_HH_
