/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef AVS_HEADER_HXX_
# define AVS_HEADER_HXX_

# include "avs_header.hh"

# include <stdexcept>
# include <cstring>

# include <wipal/tool/endianness.hh>

namespace avs
{

  inline
  size_t
  header::len_impl(size_t caplen, bool) const
  {
    if (not caplen)
      return caplen + 1;

    return tool::extract_big_endian_long_u(length);
  }

  inline
  header::time_type
  header::time_get_impl(bool) const
  {
    return mactime_get();
  }

  inline
  void
  header::time_set_impl(const time_type& t, bool)
  {
    const uint64_t	v = t.get_impl();

    if (not tool::endian::need_swap(tool::endian::little, false))
      mactime = v;
    else
      {
	const uint32_t	lo = tool::extract_swapped_long_u(v & uint32_t (-1));
	const uint32_t	hi = tool::extract_swapped_long_u(v >> 32);

	// The intent of that ``double-cast'' is to avoid GCC complaining
	// about type-punned pointers which break strict aliasing rules.

	void* const	v = static_cast<void*> (&mactime);
	uint32_t* const u = static_cast<uint32_t*> (v);

	u[0] = hi;
	u[1] = lo;
      }
  }

  inline
  bool
  header::eq_time(const pcapxx::frame_descriptor&	lhs,
		  const pcapxx::frame_descriptor&	rhs,
		  tool::endian::endianness		,
		  unsigned				prec)
  {
    const size_t		lhs_caplen = lhs.pcap_header()->caplen;
    const size_t		rhs_caplen = rhs.pcap_header()->caplen;

    const header* const		lhs_phy = (reinterpret_cast<const header*>
					   (lhs.bytes().get()));
    const header* const		rhs_phy = (reinterpret_cast<const header*>
					   (rhs.bytes().get()));

    if (lhs_phy->len(lhs_caplen, false) > lhs_caplen or
	rhs_phy->len(rhs_caplen, false) > rhs_caplen)
      return false;

    return std::abs(int64_t (lhs_phy->mactime_get() -
			     rhs_phy->mactime_get())) < prec;
  }

  inline
  uint64_t
  header::mactime_get() const
  {
    if (not tool::endian::need_swap(tool::endian::big, false))
      return mactime;
    else
      {
	// The intent of that ``double-cast'' is to avoid GCC complaining
	// about type-punned pointers which break strict aliasing rules.

	const void* const	v = static_cast<const void*> (&mactime);
	const uint32_t* const	u = static_cast<const uint32_t*> (v);

	const uint32_t	   hi = tool::extract_big_endian_long_u(u[0]);
	const uint32_t	   lo = tool::extract_big_endian_long_u(u[1]);

	return (uint64_t (hi) << 32) | lo;
      }
  }

} // End of namespace avs.

#endif // ! AVS_HEADER_HXX_
