/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef PHY_EMPTY_HH_
# define PHY_EMPTY_HH_

# include <wipal/phy/phy.hh>

// Forward declarations.
namespace phy
{
  template <class B>	struct empty_time;
  template <class B>	struct empty_header;
}

// Virtual types.
namespace tool
{

  template <class B>
  struct types< phy::empty_time<B> >
  {
    struct impl_type {};
  };

  template <class B>
  struct types< phy::empty_header<B> >
  {
    typedef phy::empty_time<bottom>	time_type;
  };

} // End of namespace tool.

// Core declarations.
namespace phy
{

  /*----------------.
  | phy::empty_time |
  `----------------*/

  /**
   * Empty timing information.
   *
   * Use when a given PHY header contains no timing
   * information. (e.g. phy::empty_header).
   */
  template <class Bottom = tool::bottom>
  struct empty_time: WP_INHERIT(public time, empty_time<Bottom>)
  {
    /// @name Static hierarchies related types and methods.
    //@{
    typedef WP_GET_EXACT(Bottom, empty_time<Bottom>)	exact_type;
    typedef WP_TYPE(impl_type, exact_type)		impl_type;

    impl_type	get_impl() const;
    exact_type&	increment(tool::microseconds microseconds);
    //@}
  };


  /*------------------.
  | phy::empty_header |
  `------------------*/

  /**
   * Empty PHY header (i.e. of length 0).
   *
   * Use when packets inside your trace contains raw IEEE 802.11 frames.
   */
  template <class Bottom = tool::bottom>
  struct empty_header: WP_INHERIT(public header, empty_header<Bottom>)
  {
    /// @name Static hierarchies related types and methods.
    //@{
    typedef WP_GET_EXACT(Bottom, empty_header<Bottom>)	exact_type;
    typedef WP_TYPE(time_type, exact_type)		time_type;

    size_t	len_impl(size_t, bool) const;
    time_type	time_get_impl(bool) const;
    void	time_set_impl(const time_type&, bool);
    static bool	eq_time(const pcapxx::frame_descriptor& lhs,
			const pcapxx::frame_descriptor& rhs,
			tool::endian::endianness	phy_end,
			unsigned			prec);
    //@}
  };

} // End of namespace phy.

# include "empty.hxx"

#endif // ! PHY_EMPTY_HH_
