/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef TOOL_DATAFILE_HH_
# define TOOL_DATAFILE_HH_

# include <string>
# include <ostream>
# include <boost/shared_ptr.hpp>

namespace tool
{

  /**
   * Output file which creation/opening is delayed until the first
   * output.
   *
   * A @c datafile object represents an output file.  It may be casted
   * to @c std::ostream& and has an @c << operator for formatted
   * outputs.  The main difference with @c std::ofstream is the file
   * is not opened until the corresponding stream is effectively
   * solicited.  Thus, if the file does not exist on the filesystem
   * and no operation is performed on it, no file will be created.
   *
   * Unlike @c std::ostream, @c datafile is copy-constructible.
   */
  struct datafile
  {
    /// Construct a @c datafile object given a filename.
    datafile(const std::string&);

    /// Retrieve the underlying C++ stream.
    operator std::ostream& ();

    /// Formatted output.
    template <class T>
    std::ostream&
    operator << (const T&);

  private:
    std::string				filename_;
    boost::shared_ptr<std::ostream>	filestream_;
  };

} // End of namespace tool.

# include "datafile.hxx"

#endif // ! TOOL_DATAFILE_HH_
