/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_INTERSECTOR_HXX_
# define WIFI_FRAME_FILTER_INTERSECTOR_HXX_

# include "intersector.hh"

# include <wipal/wifi/frame/filter/non_noisy_prism.hh>
# include <wipal/wifi/frame/filter/uniquely_identifiable.hh>

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      template <class U, class HT, template <class, class, class> class Int,
		class I1, class I2, class F>
      void
      provide_intersector(const I1& first1, const I1& last1,
			  const I2& first2, const I2& last2,
			  addr_mapping&			mapping,
			  F&				func,
			  bool				filter_prism,
			  tool::endian::endianness	phy_end)
      {
	if (filter_prism)
	  {
	    typedef non_noisy_prism<I1>				nnp1;
	    typedef non_noisy_prism<I2>				nnp2;
	    typedef typename nnp1::const_iterator		nnp1_i;
	    typedef typename nnp2::const_iterator		nnp2_i;
	    typedef uniquely_identifiable<U, nnp1_i, HT>	ui1;
	    typedef uniquely_identifiable<U, nnp2_i, HT>	ui2;
	    typedef typename ui1::const_iterator		ui1_i;
	    typedef typename ui2::const_iterator		ui2_i;
	    typedef Int<ui1_i, ui2_i, tool::bottom>		intersector;

	    nnp1	n1 (first1, last1);
	    nnp2	n2 (first2, last2);
	    ui1		u1 (n1.begin(), n1.end(), mapping, phy_end);
	    ui2		u2 (n2.begin(), n2.end(), mapping, phy_end);
	    intersector	i (u1.begin(), u1.end(), u2.begin(), u2.end());

	    func(i);
	  }
	else
	  {
	    typedef uniquely_identifiable<U, I1, HT>	ui1;
	    typedef uniquely_identifiable<U, I2, HT>	ui2;
	    typedef typename ui1::const_iterator	ui1_i;
	    typedef typename ui2::const_iterator	ui2_i;
	    typedef Int<ui1_i, ui2_i, tool::bottom>	intersector;

	    ui1		u1 (first1, last1, mapping, phy_end);
	    ui2		u2 (first2, last2, mapping, phy_end);
	    intersector	i (u1.begin(), u1.end(), u2.begin(), u2.end());

	    func(i);
	  }
      }

    } // End of namespace wifi::frame::filter.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_FILTER_INTERSECTOR_HXX_
