/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_SEQCTL_DEST_BSSID_TIMESTAMP_ID_HXX_
# define WIFI_FRAME_SEQCTL_DEST_BSSID_TIMESTAMP_ID_HXX_

# include "seqctl_dest_bssid_timestamp.hh"

# include <wipal/wifi/frame/dissector/dissector.hh>
# include <wipal/wifi/frame/unique_id/helper_hooks.hh>

namespace wifi
{

  namespace frame
  {

    template <class HeaderType>
    boost::optional<seq_dst_bss_tmp_id>
    unique_id_factory<seq_dst_bss_tmp_id>::
    build(const pcapxx::frame_descriptor&	desc,
	  addr_mapping&				mapping)
    {
      WP_UNIQUE_ID_DISSECT(desc, d, internals::id_helper_hooks, h);

      if (const timestamp* tstamp = h.timestamp_ptr())
	{
	  const addr*		da = h.addr_ptr(mgt::header::da);
	  const addr*		bssid = h.addr_ptr(mgt::header::bssid);
	  const uint16_t*	seqctl = h.seq_ctl_ptr();

	  assert(da and bssid and seqctl);

	  return seq_dst_bss_tmp_id (desc.pcap_header()->id,
				     *seqctl,
				     mapping[*da],
				     mapping[*bssid],
				     *tstamp);
	}
      return boost::none_t ();
    }

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_SEQCTL_DEST_BSSID_TIMESTAMP_ID_HXX_

