/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007, 2008  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_TIME_CONVERTER_HXX_
# define WIFI_TIME_CONVERTER_HXX_

extern "C"
{
# include <sys/time.h>
}

# include "time_converter.hh"

namespace wifi
{

  template <class PHT, class Bottom>
  time_converter<PHT, Bottom>::
  time_converter(const pcap_time_type&		local_pcap_time,
		 const phy_time_type&		local_phy_time,
		 const tool::microseconds&	global_time):
    super_type (),
    local_reference_ (local_pcap_time, local_phy_time),
    global_reference_ (global_time)
  {
  }

  template <class PHT, class Bottom>
  time_converter<PHT, Bottom>::time_values::time_values()
  {
  }

  template <class PHT, class Bottom>
  time_converter<PHT, Bottom>::
  time_values::time_values(const pcap_time_type&	pcap_time,
			   const phy_time_type&		phy_time):
    pcap_time (pcap_time),
    phy_time (phy_time)
  {
  }

  template <class PHT, class Bottom>
  typename time_converter<PHT, Bottom>::time_values
  time_converter<PHT, Bottom>::
  operator () (const tool::microseconds& global_time) const
  {
    const mpz_class	dt = global_time - global_reference_;
    struct timeval	dt_tv;
    {
      mpz_class		dsec;

      dt_tv.tv_usec = mpz_tdiv_q_ui(dsec.get_mpz_t(),
				    mpz_class (abs(dt)).get_mpz_t(),
				    1000000); // One second in microseconds.
      assert(dsec.fits_uint_p());

      dt_tv.tv_sec = dsec.get_ui();
    }

    time_values		r;

    if (sgn(dt) < 0)
      timersub(&local_reference_.pcap_time, &dt_tv, &r.pcap_time);
    else
      timeradd(&local_reference_.pcap_time, &dt_tv, &r.pcap_time);
    r.phy_time = local_reference_.phy_time + dt;
    return r;
  }

} // End of namespace wifi.

#endif // ! WIFI_TIME_CONVERTER_HXX_
