/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#include <vector>
#include <unistd.h>

#include <iostream>
#include <boost/foreach.hpp>
#include <boost/format.hpp>

#include <wipal/tool/list_of_iterables.hh>

void
fail(const std::string& s)
{
  std::cerr << "ERROR: " << s << std::endl;
  abort();
}

int
main()
{
  typedef boost::format				format;
  typedef std::vector<int>			vector;
  typedef tool::list_of_iterables<vector>	list_of_iterables;
  typedef list_of_iterables::iterator		iterator;

  const int	v1_[] = { 0, 1, 2 };
  const int	v2_[] = { 3, 4, 5 };

  const vector	v0;
  const vector	v1 (v1_, v1_ + sizeof (v1_) / sizeof (*v1_));
  const vector	v2 (v2_, v2_ + sizeof (v2_) / sizeof (*v2_));

  list_of_iterables	l0;
  list_of_iterables	l1;
  list_of_iterables	l2;

  l1.push_back(v1).push_back(v2);
  l2.push_back(v1).push_back(v0).push_back(v2);

  for (iterator i = l0.begin(); i != l0.end(); ++i)
    fail("Non-empty l0.");

  iterator		i1 = l1.begin();
  const iterator	e1 = l1.end();
  iterator		i2 = l2.begin();
  const iterator	e2 = l2.end();

  for (int i = 0; i < 6; ++i)
    {
      if (i1 == e1)
	fail("i1: unexpected end.");
      if (i2 == e2)
	fail("i2: unexpected end");
      if (*i1 != i)
	fail((format ("i1: expected %d, got %d.") % i % *i1).str());
      if (*i2 != i)
	fail((format ("i2: expected %d, got %d.") % i % *i2).str());
      ++i1;
      ++i2;
    }
  if (i1 != e1)
    fail("i1: not at end when it should be.");
  if (i2 != e2)
    fail("i2: not at end when it should be.");
}
